% Test of the decoder: BER
clear all;

% Simulation parms
Fs = 64/7; % MHz
cndbs = [2:0.1:5]; % dB
k_opt = ' -2k '; % FFT size
kr_opt = ' -m 2k '; % FFT size, new receiver
M_opt = ' -M 4 '; % mapping
a_opt = ' -a NH '; % uniformity
D_opt = ' -D 1/32 '; % cyclic prefix
p_opt = ' -p 1/2 '; % puncturing rate
I = 1; % upsampling rate

% Analysis parms
fftsize = 1024;
window = blackman(fftsize);
noverlap = 128;
randn('seed', 23525);
timepeekwin = [1:40000];

Ts = 1 / Fs;

% Transmission channels
h = [1];
%h = [1 0.1 -0.2j];
%h = [1 0 0 -0.2+0.1j 0.1 0 0 0 0 0.4j];
%load P1; tau = floor(1e6 * t / Ts) + 1; h = zeros(1, max(tau)); for aa = 1:length(tau), h(tau(aa)) = h(tau(aa)) + h_t(aa); end;
%load F1; tau = floor(1e6 * t / Ts) + 1; h = zeros(1, max(tau)); for aa = 1:length(tau), h(tau(aa)) = h(tau(aa)) + h_t(aa); end;

% Normalize energy
h = h ./ sqrt(sum(h .* conj(h)));

% Run the transmission side
dos(['dvbtenco -i test1.m2v -o test.bin -d t -v 0 -c 0 ' k_opt M_opt a_opt D_opt p_opt ' -I ' int2str(I) 'x']);

% Load the enco tx time file
fid = fopen('test.bin', 'rb');
[tx_tx, txtx_len] = fread(fid, Inf, 'float32');
fclose(fid);
tx_tx = tx_tx(1:2:end) + j.*tx_tx(2:2:end);
txtx_len = txtx_len / 2;
disp(['Read ' num2str(txtx_len) ' TX TimeXDom.']);

% Spectrum of the transmitted
[Stx_tx, ftx_tx] = psd(tx_tx, fftsize, Fs * I, window, noverlap);
avg_spectxdb = mean(10*log10(Stx_tx(1:80)));

% Find the Noise Bandwidth
Bn = 1.0 * sum(Stx_tx) * (Fs/fftsize) * 10^(-avg_spectxdb/10);

% Pass through the channel
Ptx_tx = mean(abs(tx_tx).^2)
lentx = txtx_len;
h = h ./ sqrt(sum(abs(h).^2));
rx_tx = conv(tx_tx, h); 
rx_tx = rx_tx(1:lentx);

% Cycle on the dB's
Pe = [];
for cndb = cndbs,
   
   % Add noise
   Prx_tx = mean(abs(rx_tx).^2)
   sigma = sqrt(0.5 * Prx_tx * (Fs / Bn) * 10^(-cndb/10));
   wgn = sigma .* (randn(size(rx_tx)) + j.*randn(size(rx_tx)));
   Pwgn = mean(abs(wgn).^2)
   rx_cn = rx_tx + wgn;
   rxcn_len = length(rx_cn);
   
   % Write the channeled signal
   fid = fopen('testcn.bin', 'wb');
   fwrite(fid, [real(rx_cn) imag(rx_cn)].', 'float32');
   fclose(fid);
   
   % Run the receiver
   dos(['dvbtdeco -i testcn.bin -o detest1.m2v -d g -v 1 -H lin -E zf -c 0 -s 100 ' kr_opt M_opt a_opt D_opt p_opt ' -I ' int2str(I) 'x']);
   
   % Load the enco mp mpeg2file
   fid = fopen('test1.m2v', 'rb');
   [tx_mp, txmp_len] = fread(fid, Inf, 'uchar');
   fclose(fid);
   disp(['Read ' num2str(txmp_len) ' TX MPEG-2']);
   
   % Load the deco mp mpeg2file
   fid = fopen('detest1.m2v', 'rb');
   [rx_mp, rxmp_len] = fread(fid, Inf, 'uchar');
   fclose(fid);
   disp(['Read ' num2str(rxmp_len) ' RX MPEG-2']);
   
   % Adjust lengths
   tx_mp = tx_mp(1:min(txmp_len, rxmp_len));
   rx_mp = rx_mp(1:min(txmp_len, rxmp_len));
   
   % Find the bit errors
   [biterr, Perrb] = biterr(tx_mp, rx_mp)
   %figure(10);
   %subplot(212);
   %plot(tx_mp(timepeekwin) ~= rx_mp(timepeekwin));
   %title(['Errors = ' int2str(biterr) '; BER = ' num2str(Perrb, '%.1E')]);
   %ylabel('MPEG-2 errors');
   
   Pe = [Pe Perrb];
   
end;

% plot the BER
figure(1);
semilogy(cndbs, Pe);
grid;
xlabel('C/N (dB)');
ylabel('P_e');